//@author: antokhio
//@help: template for geometry shaders
//@tags: GS, StreamOut, template
//@credits:

SamplerState s0: IMMUTABLE
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

float4x4 tW : WORLD;

struct VS_IN
{
	float4 pos : POSITION;
	float4 normal : NORMAL;
	float4 uv : TEXCOORD0;
};

struct VS_OUT
{
	float4 pos : POSITION;
	float4 normal : NORMAL;
	float4 uv : TEXCOORD0;
};

struct GS_OUT
{
	float4 pos : POSITION;
	float4 normal : NORMAL;
	float4 uv : TEXCOORD0;
};


VS_OUT VS(VS_IN input)
{
	VS_OUT output;
	
	output.pos = mul (input.pos,tW);
	output.normal = input.normal;
	output.uv = input.uv;
	
    return output;
}

[maxvertexcount(3)]
void GS(triangle VS_OUT input[3], inout TriangleStream<GS_OUT> gsout)
{ 
	GS_OUT elem = (GS_OUT)0;
	 
	float3 p1 = input[0].pos;
	float3 p2 = input[1].pos;
	float3 p3 = input[2].pos;
	
	float3 ce =( p1 + p2 + p3 )/3.0f;
	
	float3 faceEdgeA = p2 - p1;
    float3 faceEdgeB = p1 -p3;
    float3 norm = cross(faceEdgeB, faceEdgeA);	
	norm = normalize(norm);
	
	
	elem.pos = input[0].pos;
	elem.uv = input[0].uv;
	elem.normal = float4 (norm,1);
	gsout.Append(elem);
	
	elem.pos = input[1].pos;
	elem.uv = input[1].uv;
	elem.normal = float4 (norm,1);
	gsout.Append(elem);
	
	elem.pos = input[2].pos;
	elem.uv = input[2].uv;
	elem.normal = float4 (norm,1);
	gsout.Append(elem);

	gsout.RestartStrip();
}

GeometryShader StreamOutGS = ConstructGSWithSO( CompileShader( gs_4_0, GS() ), "POSITION.xyzw;NORMAL.xyzw;TEXCOORD.xyzw", NULL,NULL,NULL,-1 );
//if the above does not work, try this line instead
//GeometryShader StreamOutGS = ConstructGSWithSO( CompileShader( vs_4_0, VS() ), "POSITION.xyz;NORMAL.xyz;TEXCOORD.xy" );

technique10 PassMesh
{
    pass PP0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( StreamOutGS );
    }  
}